/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: SEMANAL_DATA_ENTR_MES                                                */
/*                                                                            */
/* Descrio:                                                                 */
/* Calcula datas de entrega para um determinado mes baseando-se na            */
/* Periodicidade de Entrega e nos Feriados de Compras.                        */
/*                                                                            */
/*                                                                            */
/* Criao:                                                                   */
/* 08/01/99 - Yoris L. de Souza                                               */
/*                                                                            */
/* Manutenes:                                                               */
/* 18/11/99 - Yoris L. de Souza                                               */
/*            Inclusao de verificacao de existencia de uma data ja calculada  */
/*            devido a um dia nao util.                                       */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* PARAMETROS:                                                                */
/*                                                                            */
/* P_CDFILIAL     - Cdigo da Filial                                          */
/* P_CDPERIENTR   - Cdigo da Periodicidade de Entrega                        */
/* P_DTINICIAL    - Data do Inicio do Mes                                     */
/* P_IDDIANUTIL   - Considera dia nao util                                    */
/******************************************************************************/
/* @D:\SQL\CALECOMP\8040_CMPSTP_SEM_DATA_ENTR_MES.SQL     */
/* @D:\SQLMAN~1\ORACLE\8040_CMPSTP_SEM_DATA_ENTR_MES.SQL */
/******************************************************************************/


CREATE OR REPLACE
PROCEDURE SF_SEMANAL_DATA_ENTR_MES
  (P_CDFILIAL     IN VARCHAR2,
   P_CDPERIENTR   IN VARCHAR2,
   P_DTINICIAL    IN DATE,
   P_IDDIANUTIL   IN VARCHAR2,
   P_DTPROCESSA   IN DATE,
   P_NRORG        IN CALECOMP.NRORG%TYPE DEFAULT 1) IS
   V_CONTAUXI   NUMBER;
   V_DIENTREGA  VARCHAR2(2);
   V_DTDIAENTR  DATE;
   V_IDDIASEM   NUMBER;
   V_DTENTRAUX  DATE;
   V_PRIMVEZ    NUMBER;
   V_JAEXISTE   NUMBER;

/*-------- CURSOR QUE CONTEM AS DATAS DE ENTREGA NO MES COM BASE NOS DIAS DE ENTREGA SEMANAIS ------*/

   CURSOR C_DATAS_ENTR IS
(SELECT NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
        FROM DIASENTR
        WHERE
         CDPERIENTR = P_CDPERIENTR AND
         NRORG      = P_NRORG      AND
         IDOCOR01   = '1' AND
         NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA
UNION
        SELECT NEXT_DAY(
         NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR02 = '1' AND
           TO_CHAR(NEXT_DAY(
            NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
           ,TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(P_DTINICIAL,'MM') AND
          ( NEXT_DAY(
         NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)
UNION
        SELECT NEXT_DAY(
         NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR03 = '1' AND
           TO_CHAR(NEXT_DAY(
            NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(P_DTINICIAL,'MM') AND
          (NEXT_DAY(NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)

UNION
        SELECT NEXT_DAY(NEXT_DAY(
         NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR04 = '1' AND
           TO_CHAR(NEXT_DAY(NEXT_DAY(
            NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(P_DTINICIAL,'MM') AND
          (NEXT_DAY(NEXT_DAY(
           NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL - 1),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)

UNION
        SELECT NEXT_DAY(LAST_DAY(P_DTINICIAL)-7,TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR05 = '1' AND
           TO_CHAR(NEXT_DAY(LAST_DAY(P_DTINICIAL)-7,TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(P_DTINICIAL,'MM') AND
            (NEXT_DAY(LAST_DAY(P_DTINICIAL)-7,TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)
             ) ORDER BY 1;

/*--- CURSOR QUE CONTEM AS DATAS DE ENTREGA DO PROXIMO MES    ------*/
/*-   COM BASE NOS DIAS DE ENTREGA SEMANAIS                   ------*/

   CURSOR C_DATAS_PROX IS
(SELECT NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
        FROM DIASENTR
        WHERE
         CDPERIENTR = P_CDPERIENTR AND
         NRORG      = P_NRORG      AND
         IDOCOR01 = '1' AND
         (NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)
UNION
        SELECT NEXT_DAY(
         NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR02 = '1' AND
           TO_CHAR(NEXT_DAY(
            NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
           ,TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(LAST_DAY(P_DTINICIAL)+1,'MM') AND
           (NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
           ,TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)
UNION
        SELECT NEXT_DAY(
         NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR03 = '1' AND
           TO_CHAR(NEXT_DAY(
            NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(LAST_DAY(P_DTINICIAL)+1,'MM') AND
          (NEXT_DAY(NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)

UNION
        SELECT NEXT_DAY(NEXT_DAY(
         NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR04 = '1' AND
           TO_CHAR(NEXT_DAY(NEXT_DAY(
            NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(LAST_DAY(P_DTINICIAL)+1,'MM') AND
          (NEXT_DAY(NEXT_DAY(NEXT_DAY(NEXT_DAY(LAST_DAY(P_DTINICIAL),TO_NUMBER(DIENTREGA))
          ,TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)),TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)

UNION
        SELECT NEXT_DAY(LAST_DAY(LAST_DAY(P_DTINICIAL)+1)-7,TO_NUMBER(DIENTREGA))
          FROM DIASENTR
           WHERE
            CDPERIENTR = P_CDPERIENTR AND
            NRORG      = P_NRORG      AND
            IDOCOR05 = '1' AND
           TO_CHAR(NEXT_DAY(LAST_DAY(LAST_DAY(P_DTINICIAL)+1),TO_NUMBER(DIENTREGA)),'MM')
           = TO_CHAR(LAST_DAY(P_DTINICIAL)+1,'MM') AND
           (NEXT_DAY(LAST_DAY(LAST_DAY(P_DTINICIAL)+1)-7,TO_NUMBER(DIENTREGA)) >= P_DTPROCESSA)
           ) ORDER BY 1;


BEGIN

    V_DTENTRAUX := P_DTINICIAL;
    OPEN C_DATAS_ENTR;
    LOOP
      FETCH C_DATAS_ENTR
      INTO V_DTDIAENTR;
      EXIT WHEN C_DATAS_ENTR%NOTFOUND;

 /*- CALCULA UMA DATA DE ENTREGA ANTERIOR A UMA DATA NAO UTIL -*/

     IF P_IDDIANUTIL = 'S' THEN
      V_CONTAUXI := 1;
       WHILE V_CONTAUXI > 0 LOOP
         SELECT COUNT(*) INTO V_CONTAUXI
         FROM FERICOMP
         WHERE
         CDFILIAL = P_CDFILIAL AND
         NRORG    = P_NRORG    AND
         DTFERICOMP = V_DTDIAENTR;
         IF V_CONTAUXI > 0 THEN
            V_DTDIAENTR := V_DTDIAENTR - 1;
         END IF;
       END LOOP;
     END IF;
/* - Verifica se j existe a data de entrega que foi calculada por um dia no util -*/
     SELECT COUNT(*) INTO V_JAEXISTE
     FROM CALECOMP
     WHERE
       CDFILIAL   = P_CDFILIAL   AND
       CDPERIENTR = P_CDPERIENTR AND
       NRORG      = P_NRORG      AND
       DTENTREGA  = V_DTDIAENTR;
/*- Se j existir a data pula -*/
     IF V_JAEXISTE = 0 THEN
   /*- VERIFICA SE JA EXISTE ALGUMA DATA DE ENTREGA CALCULADA PARA  -*/
   /*- INCLUIR A DATA ANTERIOR A PROXIMA ENTREGA  -*/

         SELECT COUNT(*) INTO V_PRIMVEZ
         FROM CALECOMP
         WHERE
         CDFILIAL   = P_CDFILIAL   AND
         CDPERIENTR = P_CDPERIENTR AND
         NRORG      = P_NRORG      AND
         DTENTREGA  = V_DTENTRAUX;

   /*- INCLUI A DATA ANTERIOR A PROXIMA ENTREGA  -*/
         IF V_PRIMVEZ > 0 THEN
            UPDATE CALECOMP SET DTANTPROXENT = V_DTDIAENTR - 1
            WHERE CDFILIAL = P_CDFILIAL AND
            CDPERIENTR = P_CDPERIENTR   AND
            NRORG      = P_NRORG        AND
            DTENTREGA = V_DTENTRAUX;

            COMMIT;
         END IF;

   /*- INCLUI A DATA DE ENTREGA CALCULADA NO CALENDARIO DE COMPRAS -*/
         INSERT INTO CALECOMP
           (CDFILIAL,
            CDPERIENTR,
            DTENTREGA,
            DTANTPROXENT,
            NRORG)
           VALUES
           (P_CDFILIAL,
            P_CDPERIENTR,
            V_DTDIAENTR,
            V_DTENTRAUX,
            P_NRORG);

         COMMIT;
     END IF;
       V_DTENTRAUX := V_DTDIAENTR;
    END LOOP;
    CLOSE C_DATAS_ENTR;

/*- PEGA A PRIMEIRA DATA ANTERIOR A PROXIMA ENTREGA  -*/

    OPEN C_DATAS_PROX;
    LOOP
      FETCH C_DATAS_PROX
      INTO V_DTENTRAUX;
      EXIT WHEN C_DATAS_PROX%NOTFOUND;

      IF P_IDDIANUTIL = 'S' THEN
          SELECT COUNT(*) INTO V_CONTAUXI
          FROM FERICOMP
          WHERE
          CDFILIAL   = P_CDFILIAL AND
          NRORG      = P_NRORG    AND
          DTFERICOMP = V_DTENTRAUX;
          IF V_CONTAUXI = 0 THEN
            EXIT;
          END IF;
      ELSE
         EXIT;
      END IF;
    END LOOP;
    CLOSE C_DATAS_PROX;

    V_DTENTRAUX := V_DTENTRAUX - 1;

/*- VERIFICA SE JA EXISTE A ULTIMA DATA DE ENTREGA QUE RECEBERA A   -*/
/*- A DATA ANTERIOR A PROXIMA ENTREGA  -*/

    SELECT COUNT(*) INTO V_PRIMVEZ
    FROM CALECOMP
    WHERE
      CDFILIAL   = P_CDFILIAL   AND
      CDPERIENTR = P_CDPERIENTR AND
      NRORG      = P_NRORG      AND
      DTENTREGA  = V_DTDIAENTR;

    IF V_PRIMVEZ > 0 THEN
/*- INCLUI A DATA ANTERIOR A PROXIMA ENTREGA PARA A A ULTIMA DATA DE ENTREGA DO MES -*/
      UPDATE CALECOMP SET DTANTPROXENT = V_DTENTRAUX
      WHERE CDFILIAL = P_CDFILIAL AND
      CDPERIENTR = P_CDPERIENTR   AND
      NRORG      = P_NRORG        AND
      DTENTREGA  = V_DTDIAENTR;

      COMMIT;
    END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('SF_SEMANAL_DATA_ENTR_MES','4.06.008'); END;
/

